/*
 * Credit to microbotlab.com
 */
#include <Servo.h>

//SERVO OBJECTS
Servo servo1;
Servo servo2;
Servo servo3;
Servo servo4;

//servo positions values, expects 1-180 deg.
int servo1Pos = 90;
int servo2Pos = 90;
int servo3Pos = 90;
int servo4Pos = 90;
//int servo5Pos = 90;//Uncoment for 5 servo arm setup
//int servo6Pos = 90;//Uncoment for 6 servo arm setup


// variables
char endOfPacketChar;//Data is sent via fixed lenght set of Integers, Terminated by a Char, Example Packet: int1,int2,int3,int4,x

// constants
const char servoEOPC = 'x';// If 'x' then serial data packet is Servo position values

//####################### SET SERVO SPEED VALUE ##################################################
//-->Adjustable Value!
const int servoSpeed = 5;// Adjustable delay in milliseconds between servo position updates, Expirement with diff values. 0= Full Speed, Higher Values = Slower Speed
//#############################################################################################

//*************** INIT AT STARTUP *******************************************************************

// Setup runs once when you press reset or power the board
void setup() {        

  // assign servo to pin numbers, Make sure servo motor cables are connected to same pins
  servo1.attach(11);
  servo2.attach(10);
  servo3.attach(9);
  servo4.attach(6);

  // initialize serial port and set baud rate
  Serial.begin(9600);// make sure same baud rate is chosen in MeCon

  Serial.print("*** MeCon 4,5,6 Servo Rbotic Arm Demo ***");   // Arduino Boot Message

}//END OF SETUP

// ******************************************************************************************************
// ********************************** MAIN PROGRAM LOOP START *******************************************
// ******************************************************************************************************

void loop() {

  //Get servo position values from serial port
  //serial in packet patern = Pos1Val,Pos2Val,Pos3Val,Pos4Val + end of packet char
  if (Serial.available() ) {
    servo1Pos = Serial.parseInt();
    servo2Pos = Serial.parseInt();
    servo3Pos = Serial.parseInt();
    servo4Pos = Serial.parseInt();
    //servo5Pos = Serial.parseInt();//Uncoment for 5 servo arm setup
    //servo6Pos = Serial.parseInt();//Uncoment for 6 servo arm setup
    endOfPacketChar = Serial.read();
  }
  
//call function to set servo positions based on recived serial data 
setServoPos(servo1,(180 - servo1Pos),servoSpeed,servoEOPC);// (180 - Servo1Pos) Example of how to Reverse rotation of servo motor
setServoPos(servo2,servo2Pos,servoSpeed,servoEOPC);
setServoPos(servo3,servo3Pos,servoSpeed,servoEOPC);
setServoPos(servo4,servo4Pos,servoSpeed,servoEOPC);
//setServoPos(servo5,servo5Pos,servoSpeed,servoEOPC);//Uncoment for 5 servo arm setup
//setServoPos(servo6,servo6Pos,servoSpeed,servoEOPC);//Uncoment for 6 servo arm setup

 
}//END OF MAIN LOOP



//********************************************************************************************************
//************************************* CUSTOM FUNCTIONS *************************************************
//********************************************************************************************************

//move servo motors to new positions 
void setServoPos(Servo servoX,int targetPosition, int speedX, char servoEOPCX){
    if (servoEOPCX != servoEOPC){
       return;
    }
    if (servoX.read()==targetPosition){
       return;
    }
    if (servoX.read() < targetPosition){
      servoX.write(servoX.read()+1);
      delay(speedX);
      return;
    }

    if (servoX.read() > targetPosition){
      servoX.write(servoX.read()-1);
      delay(speedX);
      return;
    }
}

